/*
 * Database record class for DMAP sharing
 *
 * Copyright (C) 2008 W. Michael Petullo <mike@flyn.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "dmapd-dmap-container-record.h"

/* Media ID's start at max and go down.
 * Container ID's start at 1 and go up.
 * 1 is reserved for the primary container.
 */
static gint nextid = 2;

struct DmapdDMAPContainerRecordPrivate {
	gint id;
	const char *name;
	GSList *entries;
};

gint
dmapd_dmap_container_record_get_id (DMAPContainerRecord *record)
{
	return DMAPD_DMAP_CONTAINER_RECORD (record)->priv->id;
}

const char *
dmapd_dmap_container_record_get_name (DMAPContainerRecord *record)
{
	return DMAPD_DMAP_CONTAINER_RECORD (record)->priv->name;
}

void
dmapd_dmap_container_record_add_entry (DMAPContainerRecord *record,
				       DMAPContainerEntry *entry)
{
	DMAPD_DMAP_CONTAINER_RECORD (record)->priv->entries = g_slist_append (
		DMAPD_DMAP_CONTAINER_RECORD (record)->priv->entries,
		entry);
}

GSList *
dmapd_dmap_container_record_get_entries (DMAPContainerRecord *record)
{
	return DMAPD_DMAP_CONTAINER_RECORD (record)->priv->entries;
}

static void
dmapd_dmap_container_record_init (DmapdDMAPContainerRecord *record)
{
	record->priv = DMAPD_DMAP_CONTAINER_RECORD_GET_PRIVATE (record);
	record->priv->id = nextid++;
	record->priv->entries = NULL;
}

static void
dmapd_dmap_container_record_class_init (DmapdDMAPContainerRecordClass *klass)
{
        g_type_class_add_private (klass, sizeof (DmapdDMAPContainerRecordPrivate));
}

static void
dmapd_dmap_container_record_interface_init (gpointer iface, gpointer data)
{
	DMAPContainerRecordInterface *dmap_container_record = iface;

	g_assert (G_TYPE_FROM_INTERFACE (dmap_container_record) == TYPE_DMAP_CONTAINER_RECORD);

	dmap_container_record->get_id = dmapd_dmap_container_record_get_id;
	dmap_container_record->get_name = dmapd_dmap_container_record_get_name;
	dmap_container_record->add_entry = dmapd_dmap_container_record_add_entry;
	dmap_container_record->get_entries = dmapd_dmap_container_record_get_entries;
}

G_DEFINE_TYPE_WITH_CODE (DmapdDMAPContainerRecord, dmapd_dmap_container_record, G_TYPE_OBJECT, 
			G_IMPLEMENT_INTERFACE (TYPE_DMAP_CONTAINER_RECORD,
					       dmapd_dmap_container_record_interface_init))

DmapdDMAPContainerRecord *dmapd_dmap_container_record_new (const char *name)
{
	DmapdDMAPContainerRecord *record;

	record = DMAPD_DMAP_CONTAINER_RECORD (g_object_new (TYPE_DMAPD_DMAP_CONTAINER_RECORD, NULL));

	record->priv->name = name;

	return record;
}
