/*
 *  Database class for DPAP sharing
 *
 *  Copyright (C) 2008 W. Michael Petullo <mike@flyn.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <glib/ghash.h>

#include "dmapd-dpap-db.h"
#include "dmapd-dpap-record.h"

struct DmapdDPAPDbPrivate {
	GHashTable *db;
};

DPAPRecord *dmapd_dpap_db_lookup_by_id	(DPAPDb *db, gint id)
{
	return g_hash_table_lookup (DMAPD_DPAP_DB (db)->priv->db, &id);
}

void dmapd_dpap_db_foreach_by_type	(DPAPDb *db,
				 gpointer type,
				 void (*fn) (DPAPRecord *record, gpointer data),
				 gpointer data)
{
	/* Don't use g_hash_table_foreach because we 
	 * need values, not key / value pairs. */
	GList *value = g_hash_table_get_values (DMAPD_DPAP_DB (db)->priv->db);
	g_list_foreach (value, (GFunc) fn, data);
}

gint64 dmapd_dpap_db_count_by_type	(DPAPDb *db, gpointer type)
{
	return g_hash_table_size (DMAPD_DPAP_DB (db)->priv->db);
}

void dmapd_dpap_db_add (DPAPDb *db, DPAPRecord *record)
{
	gint *id = g_new (gint, 1);
	*id = dpap_record_get_id (record);
	g_hash_table_insert (DMAPD_DPAP_DB (db)->priv->db, id, record);
}

static void dmapd_dpap_db_init (DmapdDPAPDb *db)
{
	db->priv = DMAPD_DPAP_DB_GET_PRIVATE (db);
	db->priv->db = g_hash_table_new (g_int_hash, g_int_equal);
}

static void dmapd_dpap_db_class_init (DmapdDPAPDbClass *klass)
{
	g_type_class_add_private (klass, sizeof (DmapdDPAPDbPrivate));
}

static void dmapd_dpap_db_interface_init (gpointer iface, gpointer data)
{
	DPAPDbInterface *dpap_db = iface;

	g_assert (G_TYPE_FROM_INTERFACE (dpap_db) == TYPE_DPAP_DB);

	dpap_db->lookup_by_id = dmapd_dpap_db_lookup_by_id;
	dpap_db->foreach_by_type = dmapd_dpap_db_foreach_by_type;
	dpap_db->count_by_type = dmapd_dpap_db_count_by_type;
}

G_DEFINE_TYPE_WITH_CODE (DmapdDPAPDb, dmapd_dpap_db, G_TYPE_OBJECT, 
			 G_IMPLEMENT_INTERFACE (TYPE_DPAP_DB, dmapd_dpap_db_interface_init))

DmapdDPAPDb *dmapd_dpap_db_new (void)
{
	DmapdDPAPDb *db;

	db = DMAPD_DPAP_DB (g_object_new (TYPE_DMAPD_DPAP_DB, NULL));

	return db;
}
